@@ -38,31 +38,28 @@ using namespace unity;
 
 namespace scopes_ng {
 
+const int MAX_NUMBER_OF_CATEGORIES = 32; // when reached, any excess categories which have no results will be removed
+
 // FIXME: this should be in a common place
 #define CATEGORY_JSON_DEFAULTS R"({"schema-version":1,"template": {"category-layout":"grid","card-layout":"vertical","card-size":"small","overlay-mode":null,"collapsed-rows":2}, "components": { "title":null, "art": { "aspect-ratio":1.0 }, "subtitle":null, "mascot":null, "emblem":null, "summary":null, "attributes": { "max-count":2 }, "background":null, "overlay-color":null }, "resources":{}})"
 
 class CategoryData
 {
 public:
-    CategoryData(scopes::Category::SCPtr const& category): m_resultsModel(nullptr), m_isSpecial(false)
+    CategoryData(scopes::Category::SCPtr const& category): m_isSpecial(false)
     {
         setCategory(category);
     }
 
+    CategoryData(CategoryData const& other) = delete;
+
     // constructor for special (shell-overriden) categories
-    CategoryData(QString const& id, QString const& title, QString const& icon, QString rawTemplate, QObject* countObject):
-        m_catId(id), m_catTitle(title), m_catIcon(icon), m_rawTemplate(rawTemplate.toStdString()), m_resultsModel(nullptr), m_countObject(countObject), m_isSpecial(true)
+    CategoryData(QString const& id, QString const& title, QString const& icon, QString const& rawTemplate, QObject* countObject):
+        m_catId(id), m_catTitle(title), m_catIcon(icon), m_rawTemplate(rawTemplate.toStdString()), m_countObject(countObject), m_isSpecial(true)
     {
         parseTemplate(m_rawTemplate, &m_rendererTemplate, &m_components);
     }
 
-    ~CategoryData()
-    {
-        if (m_resultsModel) {
-            delete m_resultsModel;
-        }
-    }
-
     void setCategory(scopes::Category::SCPtr const& category)
     {
         m_category = category;
@@ -132,7 +129,7 @@ public:
         for (auto it = components_dict.begin(); it != components_dict.end(); ++it) {
             if (it.value().isObject() == false) continue;
             QJsonObject component_dict(it.value().toObject());
-            QString fieldName(component_dict.value("field").toString());
+            QString fieldName(component_dict.value(QStringLiteral("field")).toString());
             if (fieldName.isEmpty()) continue;
             result[it.key()] = fieldName;
         }
@@ -143,13 +140,13 @@ public:
     int getMaxAttributes() const
     {
         QJsonObject components_obj = m_components.toObject();
-        QJsonObject attrs_obj = components_obj.value("attributes").toObject();
-        QJsonValue max_count_val = attrs_obj.value("max-count");
+        QJsonObject attrs_obj = components_obj.value(QStringLiteral("attributes")).toObject();
+        QJsonValue max_count_val = attrs_obj.value(QStringLiteral("max-count"));
 
         return max_count_val.toInt(2);
     }
 
-    QVector<int> updateAttributes(scopes::Category::SCPtr category)
+    QVector<int> updateAttributes(const scopes::Category::SCPtr& category)
     {
         QVector<int> roles;
 
@@ -191,12 +188,12 @@ public:
         return roles;
     }
 
-    void setResultsModel(ResultsModel* model)
+    void setResultsModel(const QSharedPointer<ResultsModel>& model)
     {
         m_resultsModel = model;
     }
 
-    ResultsModel* resultsModel() const
+    QSharedPointer<ResultsModel> resultsModel() const
     {
         return m_resultsModel;
     }
@@ -235,10 +232,10 @@ public:
 
         QJsonObject category_root = mergeOverrides(*DEFAULTS, category_doc.object()).toObject();
         // fixup parts we mangle
-        QJsonValueRef templateRef = category_root["template"];
+        QJsonValueRef templateRef = category_root[QStringLiteral("template")];
         QJsonObject templateObj(templateRef.toObject());
-        if (templateObj.contains("card-background")) {
-            QJsonValueRef cardBackgroundRef = templateObj["card-background"];
+        if (templateObj.contains(QStringLiteral("card-background"))) {
+            QJsonValueRef cardBackgroundRef = templateObj[QStringLiteral("card-background")];
             if (cardBackgroundRef.isString()) {
                 QString background(cardBackgroundRef.toString());
                 cardBackgroundRef = QJsonValue::fromVariant(backgroundUriToVariant(background));
@@ -246,22 +243,22 @@ public:
             }
         }
         // FIXME: validate the merged json
-        *renderer = category_root.value(QString("template"));
-        *components = category_root.value(QString("components"));
+        *renderer = category_root.value(QStringLiteral("template"));
+        *components = category_root.value(QStringLiteral("components"));
 
         return true;
     }
 
+    scopes::Category::SCPtr m_category;
 private:
     static QJsonValue* DEFAULTS;
-    scopes::Category::SCPtr m_category;
     QString m_catId;
     QString m_catTitle;
     QString m_catIcon;
     std::string m_rawTemplate;
     QJsonValue m_rendererTemplate;
     QJsonValue m_components;
-    ResultsModel* m_resultsModel;
+    QSharedPointer<ResultsModel> m_resultsModel;
     QPointer<QObject> m_countObject;
     bool m_isSpecial;
 
@@ -289,7 +286,7 @@ private:
         } else if (overrideVal.isString() && (defaultVal.isNull() || defaultVal.isObject())) {
             // special case the expansion of "art": "icon" -> "art": {"field": "icon"}
             QJsonObject resultObj(defaultVal.toObject());
-            resultObj.insert("field", overrideVal);
+            resultObj.insert(QStringLiteral("field"), overrideVal);
             return resultObj;
         } else if (defaultVal.isNull() && overrideVal.isObject()) {
             return overrideVal;
@@ -306,6 +303,12 @@ Categories::Categories(QObject* parent)
 {
 }
 
+Categories::~Categories()
+{
+    m_categories.clear();
+    m_categoryResults.clear();
+}
+
 int Categories::rowCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent);
@@ -313,7 +316,7 @@ int Categories::rowCount(const QModelIndex& parent) const
     return m_categories.size();
 }
 
-ResultsModel* Categories::lookupCategory(std::string const& category_id)
+QSharedPointer<ResultsModel> Categories::lookupCategory(std::string const& category_id)
 {
     return m_categoryResults[category_id];
 }
@@ -344,7 +347,7 @@ int Categories::getFirstEmptyCategoryIndex() const
     return m_categories.size();
 }
 
-void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel* resultsModel)
+void Categories::registerCategory(const scopes::Category::SCPtr& category, QSharedPointer<ResultsModel> resultsModel)
 {
     // do we already have a category with this id?
     int index = getCategoryIndex(QString::fromStdString(category->id()));
@@ -371,7 +374,7 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
             m_categories.insert(emptyIndex, catData);
             endInsertRows();
         } else {
-            CategoryData* catData = m_categories[index].data();
+            QSharedPointer<CategoryData> catData = m_categories[index];
             // check if any attributes of the category changed
             QVector<int> changedRoles(catData->updateAttributes(category));
 
@@ -386,15 +389,15 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
             }
         }
     } else {
-        CategoryData* catData = new CategoryData(category);
-        if (resultsModel == nullptr) {
-            resultsModel = new ResultsModel(this);
+        QSharedPointer<CategoryData> catData(new CategoryData(category));
+        if (!resultsModel) {
+            resultsModel.reset(new ResultsModel());
         }
         catData->setResultsModel(resultsModel);
 
         beginInsertRows(QModelIndex(), emptyIndex, emptyIndex);
 
-        m_categories.insert(emptyIndex, QSharedPointer<CategoryData>(catData));
+        m_categories.insert(emptyIndex, catData);
         resultsModel->setCategoryId(QString::fromStdString(category->id()));
         resultsModel->setComponentsMapping(catData->getComponentsMapping());
         resultsModel->setMaxAtrributesCount(catData->getMaxAttributes());
@@ -402,9 +405,29 @@ void Categories::registerCategory(scopes::Category::SCPtr category, ResultsModel
 
         endInsertRows();
     }
+
+    if (m_categories.count() >= MAX_NUMBER_OF_CATEGORIES) {
+        // we register one category at a time, so there can be one excess category at most
+        const int index = m_categories.count() - 1;
+        auto it = m_categories.begin() + index;
+        if ((*it)->resultsModelCount() == 0) {
+            qDebug() << "Purging unused category:" << (*it)->categoryId();
+            beginRemoveRows(QModelIndex(), index, index);
+            m_categoryResults.remove((*it)->categoryId().toStdString());
+            for (auto kv = m_countObjects.begin(); kv != m_countObjects.end(); ++kv) {
+                if (kv.value() == (*it)->categoryId()) {
+                    kv.key()->deleteLater();
+                    m_countObjects.erase(kv);
+                    break;
+                }
+            }
+            m_categories.erase(it);
+            endRemoveRows();
+        }
+    }
 }
 
-void Categories::updateResultCount(ResultsModel* resultsModel)
+void Categories::updateResultCount(const QSharedPointer<ResultsModel>& resultsModel)
 {
     int idx = -1;
     for (int i = 0; i < m_categories.count(); i++) {
@@ -429,7 +452,7 @@ void Categories::clearAll()
 {
     if (m_categories.count() == 0) return;
 
-    Q_FOREACH(ResultsModel* model, m_categoryResults) {
+    Q_FOREACH(QSharedPointer<ResultsModel> model, m_categoryResults) {
         model->clearResults();
     }
 
@@ -450,7 +473,7 @@ bool Categories::overrideCategoryJson(QString const& categoryId, QString const&
 {
     int idx = getCategoryIndex(categoryId);
     if (idx >= 0) {
-        CategoryData* catData = m_categories.at(idx).data();
+        QSharedPointer<CategoryData> catData = m_categories.at(idx);
         if (!catData->overrideTemplate(json.toStdString())) {
             return false;
         }
@@ -477,10 +500,10 @@ void Categories::addSpecialCategory(QString const& categoryId, QString const& na
     if (index >= 0) {
         qWarning("ERROR! Category with id \"%s\" already exists!", categoryId.toStdString().c_str());
     } else {
-        CategoryData* catData = new CategoryData(categoryId, name, icon, rawTemplate, countObject);
+        QSharedPointer<CategoryData> catData(new CategoryData(categoryId, name, icon, rawTemplate, countObject));
         // prepend the category
         beginInsertRows(QModelIndex(), 0, 0);
-        m_categories.prepend(QSharedPointer<CategoryData>(catData));
+        m_categories.prepend(catData);
         endInsertRows();
 
         if (countObject) {
@@ -510,8 +533,22 @@ void Categories::countChanged()
 QVariant
 Categories::data(const QModelIndex& index, int role) const
 {
-    CategoryData* catData = m_categories.at(index.row()).data();
-    ResultsModel* resultsModel = catData->resultsModel();
+    int row = index.row();
+    if (row >= m_categories.size() || row < 0)
+    {
+        qWarning() << "Categories::data - invalid index" << row << "size"
+                << m_categories.size();
+        return QVariant();
+    }
+
+    QSharedPointer<CategoryData> catData = m_categories.at(row);
+
+    if (!catData)
+    {
+        qWarning() << "Categories::data - invalid category data at" << row << "size"
+                        << m_categories.size();
+        return QVariant();
+    }
 
     switch (role) {
         case RoleCategoryId:
@@ -529,9 +566,43 @@ Categories::data(const QModelIndex& index, int role) const
         case RoleHeaderLink:
             return catData->headerLink();
         case RoleResults:
-            return QVariant::fromValue(resultsModel);
+        {
+            QSharedPointer<ResultsModel> resultsModel = catData->resultsModel();
+            if (resultsModel)
+            {
+                return QVariant::fromValue(resultsModel.data());
+            }
+            else
+            {
+                qWarning() << "Category data has no results model" << catData->categoryId();
+                return QVariant();
+            }
+        }
         case RoleCount:
             return catData->resultsModelCount();
+        case RoleResultsSPtr:
+        {
+            QSharedPointer<unity::shell::scopes::ResultsModelInterface> resultsModel = catData->resultsModel();
+            if (resultsModel)
+            {
+                return QVariant::fromValue(resultsModel);
+            }
+            else
+            {
+                return QVariant();
+            }
+        }
+        case RoleCategorySPtr:
+        {
+            if (catData->m_category)
+            {
+                return QVariant::fromValue(catData->m_category);
+            }
+            else
+            {
+                return QVariant();
+            }
+        }
         default:
             return QVariant();
     }
