@@ -34,7 +34,7 @@
 namespace scopes_ng
 {
 
-struct CategoryData;
+class CategoryData;
 
 class Q_DECL_EXPORT Categories : public unity::shell::scopes::CategoriesInterface
 {
@@ -43,15 +43,23 @@ class Q_DECL_EXPORT Categories : public unity::shell::scopes::CategoriesInterfac
 public:
     explicit Categories(QObject* parent = 0);
 
+    enum ExtraRoles
+    {
+        RoleResultsSPtr = 999998,
+        RoleCategorySPtr = 999999
+    };
+
+    ~Categories();
+
     QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
     int rowCount(const QModelIndex& parent = QModelIndex()) const override;
 
     Q_INVOKABLE bool overrideCategoryJson(QString const& categoryId, QString const& json) override;
     Q_INVOKABLE void addSpecialCategory(QString const& categoryId, QString const& name, QString const& icon, QString const& rawTemplate, QObject* countObject) override;
 
-    ResultsModel* lookupCategory(std::string const& category_id);
-    void registerCategory(unity::scopes::Category::SCPtr category, ResultsModel* model);
-    void updateResultCount(ResultsModel* resultsModel);
+    QSharedPointer<ResultsModel> lookupCategory(std::string const& category_id);
+    void registerCategory(const unity::scopes::Category::SCPtr& category, QSharedPointer<ResultsModel> model);
+    void updateResultCount(const QSharedPointer<ResultsModel>& resultsModel);
     void clearAll();
 
     static bool parseTemplate(std::string const& raw_template, QJsonValue* renderer, QJsonValue* components);
@@ -64,12 +72,13 @@ private:
     int getFirstEmptyCategoryIndex() const;
 
     QList<QSharedPointer<CategoryData>> m_categories;
-    QMap<std::string, ResultsModel*> m_categoryResults;
+    QMap<std::string, QSharedPointer<ResultsModel>> m_categoryResults;
     QMap<QObject*, QString> m_countObjects;
 };
 
 } // namespace scopes_ng
 
 Q_DECLARE_METATYPE(scopes_ng::Categories*)
+Q_DECLARE_METATYPE(std::shared_ptr<const unity::scopes::Category>)
 
 #endif // NG_CATEGORIES_H
