@@ -73,12 +73,13 @@ public:
 
     enum Type { SEARCH, PREVIEW, ACTIVATION };
 
-    PushEvent(Type event_type, std::shared_ptr<CollectorBase> collector);
+    PushEvent(Type event_type, const std::shared_ptr<CollectorBase>& collector);
     Type type();
 
     CollectorBase::Status collectSearchResults(QList<std::shared_ptr<unity::scopes::CategorisedResult>>& out_results, unity::scopes::Department::SCPtr& out_rootDepartment, unity::scopes::OptionSelectorFilter::SCPtr& out_sortOrder, unity::scopes::FilterState& out_filterState);
     CollectorBase::Status collectPreviewData(unity::scopes::ColumnLayoutList& out_columns, unity::scopes::PreviewWidgetList& out_widgets, QHash<QString, QVariant>& out_data);
-    CollectorBase::Status collectActivationResponse(std::shared_ptr<unity::scopes::ActivationResponse>& out_response, std::shared_ptr<unity::scopes::Result>& out_result);
+    CollectorBase::Status collectActivationResponse(std::shared_ptr<unity::scopes::ActivationResponse>& out_response, std::shared_ptr<unity::scopes::Result>&
+            out_result, QString& categoryId);
 
     qint64 msecsSinceStart() const;
 
@@ -137,10 +138,11 @@ public:
     virtual void activated(unity::scopes::ActivationResponse const&) override;
     virtual void finished(unity::scopes::CompletionDetails const& details) override;
 
-    ActivationReceiver(QObject* receiver, std::shared_ptr<unity::scopes::Result> const& result);
+    ActivationReceiver(QObject* receiver, std::shared_ptr<unity::scopes::Result> const& result, QString const& categoryId = "");
 
 private:
     std::shared_ptr<ActivationCollector> m_collector;
+    QString m_categoryId;
 };
 
 } // namespace scopes_ng
