@@ -33,7 +33,6 @@
 #include <QUrl>
 #include <QUrlQuery>
 #include <QDebug>
-#include <QGSettings>
 #include <QtGui/QDesktopServices>
 #include <QEvent>
 #include <QMutex>
@@ -81,7 +80,7 @@ Scope::Ptr Scope::newInstance(scopes_ng::Scopes* parent)
 
 Scope::Scope(scopes_ng::Scopes* parent) :
       m_query_id(0)
-    , m_formFactor("phone")
+    , m_formFactor(QStringLiteral("phone"))
     , m_isActive(false)
     , m_searchInProgress(false)
     , m_resultsDirty(false)
@@ -97,9 +96,6 @@ Scope::Scope(scopes_ng::Scopes* parent) :
     QQmlEngine::setObjectOwnership(this, QQmlEngine::CppOwnership);
     m_categories.reset(new Categories(this));
 
-    m_settings = QGSettings::isSchemaInstalled("com.canonical.Unity.Lenses") ? new QGSettings("com.canonical.Unity.Lenses", QByteArray(), this) : nullptr;
-    QObject::connect(m_settings, &QGSettings::changed, this, &Scope::internetFlagChanged);
-
     setScopesInstance(parent);
 
     m_typingTimer.setSingleShot(true);
@@ -228,6 +224,9 @@ void Scope::handleActivation(std::shared_ptr<scopes::ActivationResponse> const&
          case scopes::ActivationResponse::PerformQuery:
             executeCannedQuery(response->query(), true);
             break;
+        case scopes::ActivationResponse::UpdatePreview:
+            handlePreviewUpdate(result, response->updated_widgets());
+            break;
         default:
             break;
     }
@@ -235,6 +234,11 @@ void Scope::handleActivation(std::shared_ptr<scopes::ActivationResponse> const&
 
 void Scope::metadataRefreshed()
 {
+    // refresh Settings view if needed
+    if (require_child_scopes_refresh()) {
+        update_child_scopes();
+    }
+
     std::shared_ptr<scopes::ActivationResponse> response;
     response.swap(m_delayedActivation);
 
@@ -247,13 +251,33 @@ void Scope::metadataRefreshed()
     }
 }
 
-void Scope::internetFlagChanged(QString const& key)
+void Scope::setCannedQuery(unity::scopes::CannedQuery const& query)
 {
-    if (key != "remoteContentSearch") {
-        return;
+    setCurrentNavigationId(QString::fromStdString(query.department_id()));
+    setFilterState(query.filter_state());
+    if (query.has_user_data()) {
+        m_queryUserData.reset(new unity::scopes::Variant(query.user_data()));
     }
+    else
+    {
+        m_queryUserData.reset(nullptr);
+    }
+    setSearchQueryString(QString::fromStdString(query.query_string()));
+}
 
-    invalidateResults();
+void Scope::handlePreviewUpdate(unity::scopes::Result::SPtr const& result, unity::scopes::PreviewWidgetList const& widgets)
+{
+    for (auto stack: m_previewStacks) {
+        auto previewedResult = stack->previewedResult();
+
+        if (result == nullptr) {
+            qWarning() << "handlePreviewUpdate: result is null";
+            return;
+        }
+        if (previewedResult != nullptr && *result == *previewedResult) {
+            stack->update(widgets);
+        }
+    }
 }
 
 void Scope::executeCannedQuery(unity::scopes::CannedQuery const& query, bool allowDelayedActivation)
@@ -263,9 +287,7 @@ void Scope::executeCannedQuery(unity::scopes::CannedQuery const& query, bool all
         return;
     }
 
-    QString scopeId(QString::fromStdString(query.scope_id()));
-    QString searchString(QString::fromStdString(query.query_string()));
-    QString departmentId(QString::fromStdString(query.department_id()));
+    const QString scopeId(QString::fromStdString(query.scope_id()));
 
     Scope* scope = nullptr;
     if (scopeId == id()) {
@@ -279,15 +301,15 @@ void Scope::executeCannedQuery(unity::scopes::CannedQuery const& query, bool all
     }
 
     if (scope) {
-        scope->setCurrentNavigationId(departmentId);
-        scope->setFilterState(query.filter_state());
-        scope->setSearchQuery(searchString);
+        scope->setCannedQuery(query);
         // FIXME: implement better way to do multiple changes to search props and dispatch single search
         if (!scope->searchInProgress()) {
             scope->invalidateResults();
         }
         if (scope != this) {
             Q_EMIT gotoScope(scopeId);
+        } else {
+            Q_EMIT showDash();
         }
     } else {
         // create temp dash page
@@ -295,9 +317,7 @@ void Scope::executeCannedQuery(unity::scopes::CannedQuery const& query, bool all
         if (meta_sptr) {
             Scope::Ptr newScope = Scope::newInstance(m_scopesInstance);
             newScope->setScopeData(*meta_sptr);
-            newScope->setCurrentNavigationId(departmentId);
-            newScope->setFilterState(query.filter_state());
-            newScope->setSearchQuery(searchString);
+            newScope->setCannedQuery(query);
             m_scopesInstance->addTempScope(newScope);
             Q_EMIT openScope(newScope.data());
         } else if (allowDelayedActivation) {
@@ -380,7 +400,7 @@ void Scope::flushUpdates(bool finalize)
         bool containsDepartments = m_rootDepartment.get() != nullptr;
         // design decision - no navigation when doing searches
         containsDepartments &= m_searchQuery.isEmpty();
-        
+
         if (containsDepartments != m_hasNavigation) {
             m_hasNavigation = containsDepartments;
             Q_EMIT hasNavigationChanged();
@@ -388,7 +408,7 @@ void Scope::flushUpdates(bool finalize)
 
         if (!containsDepartments && !m_currentNavigationId.isEmpty()) {
             qDebug() << "Resetting current nav id";
-            m_currentNavigationId = "";
+            m_currentNavigationId = QLatin1String("");
             Q_EMIT currentNavigationIdChanged();
         }
     }
@@ -429,7 +449,7 @@ void Scope::flushUpdates(bool finalize)
 
         if (!containsAltNav && !m_currentAltNavigationId.isEmpty()) {
             qDebug() << "Resetting alt nav id";
-            m_currentAltNavigationId = "";
+            m_currentAltNavigationId = QLatin1String("");
             Q_EMIT currentAltNavigationIdChanged();
         }
 
@@ -479,7 +499,9 @@ scopes::Department::SCPtr Scope::findUpdateNode(DepartmentNode* node, scopes::De
 
     // are all the children in our cache?
     QStringList cachedChildrenIds;
-    Q_FOREACH(DepartmentNode* child, node->childNodes()) {
+    const auto childNodes = node->childNodes();
+    cachedChildrenIds.reserve(childNodes.count());
+    Q_FOREACH(DepartmentNode* child, childNodes) {
         cachedChildrenIds << child->id();
     }
 
@@ -544,7 +566,7 @@ void Scope::processResultSet(QList<std::shared_ptr<scopes::CategorisedResult>>&
     if (result_set.count() == 0) return;
 
     // this will keep the list of categories in order
-    QList<scopes::Category::SCPtr> categories;
+    QVector<scopes::Category::SCPtr> categories;
 
     // split the result_set by category_id
     QMap<std::string, QList<std::shared_ptr<scopes::CategorisedResult>>> category_results;
@@ -707,16 +729,10 @@ void Scope::dispatchSearch()
             meta["session-id"] = uuidToString(m_session_id).toStdString();
         }
         meta["query-id"] = unity::scopes::Variant(m_query_id);
-        if (m_settings) {
-            QVariant remoteSearch(m_settings->get("remote-content-search"));
-            if (remoteSearch.toString() == QString("none")) {
-                meta["no-internet"] = true;
-            }
-        }
         try {
             if (m_settingsModel && m_scopeMetadata && m_scopeMetadata->location_data_needed())
             {
-                QVariant locationEnabled = m_settingsModel->value("internal.location");
+                QVariant locationEnabled = m_settingsModel->value(QStringLiteral("internal.location"));
                 if (locationEnabled.type() == QVariant::Bool && locationEnabled.toBool())
                 {
                     meta.set_location(m_locationService->location());
@@ -732,7 +748,9 @@ void Scope::dispatchSearch()
         m_searchController->setListener(listener);
         try {
             qDebug() << "Dispatching search:" << id() << m_searchQuery << m_currentNavigationId;
-            scopes::QueryCtrlProxy controller = m_proxy->search(m_searchQuery.toStdString(), m_currentNavigationId.toStdString(), m_filterState, meta, listener);
+            scopes::QueryCtrlProxy controller = m_queryUserData ?
+                m_proxy->search(m_searchQuery.toStdString(), m_currentNavigationId.toStdString(), m_filterState, *m_queryUserData, meta, listener) :
+                m_proxy->search(m_searchQuery.toStdString(), m_currentNavigationId.toStdString(), m_filterState, meta, listener);
             m_searchController->setController(controller);
         } catch (std::exception& e) {
             qWarning("Caught an error from create_query(): %s", e.what());
@@ -767,7 +785,7 @@ void Scope::setScopeData(scopes::ScopeMetadata const& data)
         }
         else
         {
-            shareDir = QDir::home().filePath(".config/unity-scopes");
+            shareDir = QDir::home().filePath(QStringLiteral(".config/unity-scopes"));
         }
 
         m_settingsModel.reset(
@@ -869,6 +887,23 @@ unity::shell::scopes::SettingsModelInterface* Scope::settings() const
     return m_settingsModel.data();
 }
 
+bool Scope::require_child_scopes_refresh() const
+{
+    if (m_settingsModel && m_scopesInstance)
+    {
+        return m_settingsModel->require_child_scopes_refresh();
+    }
+    return false;
+}
+
+void Scope::update_child_scopes()
+{
+    if (m_settingsModel && m_scopesInstance)
+    {
+        m_settingsModel->update_child_scopes(m_scopesInstance->getAllMetadata());
+    }
+}
+
 /*
 Filters* Scope::filters() const
 {
@@ -958,6 +993,17 @@ void Scope::departmentModelDestroyed(QObject* obj)
     m_inverseDepartments.erase(it);
 }
 
+void Scope::previewStackDestroyed(QObject *obj)
+{
+    for (auto it = m_previewStacks.begin(); it != m_previewStacks.end(); it++)
+    {
+        if (*it == obj) {
+            m_previewStacks.erase(it);
+            break;
+        }
+    }
+}
+
 void Scope::performQuery(QString const& cannedQuery)
 {
     try {
@@ -1020,6 +1066,16 @@ QVariantMap Scope::customizations() const
 }
 
 void Scope::setSearchQuery(const QString& search_query)
+{
+    // this method is called by the shell when user types in search string,
+    // it needs to reset canned query user data.
+    if (m_searchQuery.isNull() || search_query != m_searchQuery) {
+        m_queryUserData.reset(nullptr);
+    }
+    setSearchQueryString(search_query);
+}
+
+void Scope::setSearchQueryString(const QString& search_query)
 {
     /* Checking for m_searchQuery.isNull() which returns true only when the string
        has never been set is necessary because when search_query is the empty
@@ -1108,7 +1164,7 @@ void Scope::setFavorite(const bool value)
     }
 }
 
-void Scope::activate(QVariant const& result_var)
+void Scope::activate(QVariant const& result_var, QString const& categoryId)
 {
     if (!result_var.canConvert<std::shared_ptr<scopes::Result>>()) {
         qWarning("Cannot activate, unable to convert %s to Result", result_var.typeName());
@@ -1124,18 +1180,18 @@ void Scope::activate(QVariant const& result_var)
     if (result->contains("online_account_details"))
     {
         QVariantMap details = scopeVariantToQVariant(result->value("online_account_details")).toMap();
-        if (details.contains("service_name") &&
-            details.contains("service_type") &&
-            details.contains("provider_name") &&
-            details.contains("login_passed_action") &&
-            details.contains("login_failed_action"))
+        if (details.contains(QStringLiteral("service_name")) &&
+            details.contains(QStringLiteral("service_type")) &&
+            details.contains(QStringLiteral("provider_name")) &&
+            details.contains(QStringLiteral("login_passed_action")) &&
+            details.contains(QStringLiteral("login_failed_action")))
         {
-            bool success = loginToAccount(details.contains("scope_id") ? details.value("scope_id").toString() : "",
-                                          details.value("service_name").toString(),
-                                          details.value("service_type").toString(),
-                                          details.value("provider_name").toString());
+            bool success = loginToAccount(details.contains(QStringLiteral("scope_id")) ? details.value(QStringLiteral("scope_id")).toString() : QLatin1String(""),
+                                          details.value(QStringLiteral("service_name")).toString(),
+                                          details.value(QStringLiteral("service_type")).toString(),
+                                          details.value(QStringLiteral("provider_name")).toString());
 
-            int action_code_index = success ? details.value("login_passed_action").toInt() : details.value("login_failed_action").toInt();
+            int action_code_index = success ? details.value(QStringLiteral("login_passed_action")).toInt() : details.value(QStringLiteral("login_failed_action")).toInt();
             if (action_code_index >= 0 && action_code_index <= scopes::OnlineAccountClient::LastActionCode_)
             {
                 scopes::OnlineAccountClient::PostLoginAction action_code = static_cast<scopes::OnlineAccountClient::PostLoginAction>(action_code_index);
@@ -1154,8 +1210,12 @@ void Scope::activate(QVariant const& result_var)
     }
 
     if (result->direct_activation()) {
-        activateUri(QString::fromStdString(result->uri()));
-    } else {
+        if (result->uri().find("scope://") == 0 || id() == QLatin1String("clickscope") || (id() == QLatin1String("videoaggregator") && categoryId == QLatin1String("myvideos-getstarted"))) {
+            activateUri(QString::fromStdString(result->uri()));
+        } else {
+            Q_EMIT previewRequested(result_var);
+        }
+    } else { // intercept activation flag set
         try {
             cancelActivation();
             scopes::ActivationListenerBase::SPtr listener(new ActivationReceiver(this, result));
@@ -1173,7 +1233,7 @@ void Scope::activate(QVariant const& result_var)
     }
 }
 
-unity::shell::scopes::PreviewStackInterface* Scope::preview(QVariant const& result_var)
+unity::shell::scopes::PreviewStackInterface* Scope::preview(QVariant const& result_var, QString const& categoryId)
 {
     if (!result_var.canConvert<std::shared_ptr<scopes::Result>>()) {
         qWarning("Cannot preview, unable to convert %s to Result", result_var.typeName());
@@ -1186,21 +1246,26 @@ unity::shell::scopes::PreviewStackInterface* Scope::preview(QVariant const& resu
         return nullptr;
     }
 
+    // No preview for scope:// uris and for special camera-app card in video aggregator scope (if no videos are available).
+    if (result->uri().find("scope://") == 0 || (id() == QLatin1String("videoaggregator") && categoryId == QLatin1String("myvideos-getstarted"))) {
+        return nullptr;
+    }
+
     if (result->contains("online_account_details"))
     {
         QVariantMap details = scopeVariantToQVariant(result->value("online_account_details")).toMap();
-        if (details.contains("service_name") &&
-            details.contains("service_type") &&
-            details.contains("provider_name") &&
-            details.contains("login_passed_action") &&
-            details.contains("login_failed_action"))
+        if (details.contains(QStringLiteral("service_name")) &&
+            details.contains(QStringLiteral("service_type")) &&
+            details.contains(QStringLiteral("provider_name")) &&
+            details.contains(QStringLiteral("login_passed_action")) &&
+            details.contains(QStringLiteral("login_failed_action")))
         {
-            bool success = loginToAccount(details.contains("scope_id") ? details.value("scope_id").toString() : "",
-                                          details.value("service_name").toString(),
-                                          details.value("service_type").toString(),
-                                          details.value("provider_name").toString());
+            bool success = loginToAccount(details.contains(QStringLiteral("scope_id")) ? details.value(QStringLiteral("scope_id")).toString() : QLatin1String(""),
+                                          details.value(QStringLiteral("service_name")).toString(),
+                                          details.value(QStringLiteral("service_type")).toString(),
+                                          details.value(QStringLiteral("provider_name")).toString());
 
-            int action_code_index = success ? details.value("login_passed_action").toInt() : details.value("login_failed_action").toInt();
+            int action_code_index = success ? details.value(QStringLiteral("login_passed_action")).toInt() : details.value(QStringLiteral("login_failed_action")).toInt();
             if (action_code_index >= 0 && action_code_index <= scopes::OnlineAccountClient::LastActionCode_)
             {
                 scopes::OnlineAccountClient::PostLoginAction action_code = static_cast<scopes::OnlineAccountClient::PostLoginAction>(action_code_index);
@@ -1219,6 +1284,8 @@ unity::shell::scopes::PreviewStackInterface* Scope::preview(QVariant const& resu
     }
 
     PreviewStack* stack = new PreviewStack(nullptr);
+    QObject::connect(stack, &QObject::destroyed, this, &Scope::previewStackDestroyed);
+    m_previewStacks.append(stack);
     stack->setAssociatedScope(this, m_session_id, m_scopesInstance->userAgentString());
     stack->loadForResult(result);
     return stack;
