@@ -43,8 +43,6 @@
 #include "department.h"
 #include "locationservice.h"
 
-class QGSettings;
-
 namespace scopes_ng
 {
 
@@ -130,6 +128,8 @@ public:
     unity::shell::scopes::ScopeInterface::Status status() const override;
     unity::shell::scopes::CategoriesInterface* categories() const override;
     unity::shell::scopes::SettingsModelInterface* settings() const override;
+    bool require_child_scopes_refresh() const;
+    void update_child_scopes();
     QString searchQuery() const override;
     QString noResultsHint() const override;
     QString formFactor() const override;
@@ -147,8 +147,8 @@ public:
     void setActive(const bool) override;
     void setFavorite(const bool) override;
 
-    Q_INVOKABLE void activate(QVariant const& result) override;
-    Q_INVOKABLE unity::shell::scopes::PreviewStackInterface* preview(QVariant const& result) override;
+    Q_INVOKABLE void activate(QVariant const& result, QString const& categoryId) override;
+    Q_INVOKABLE unity::shell::scopes::PreviewStackInterface* preview(QVariant const& result, QString const& categoryId) override;
     Q_INVOKABLE void cancelActivation() override;
     Q_INVOKABLE void closeScope(unity::shell::scopes::ScopeInterface* scope) override;
     Q_INVOKABLE unity::shell::scopes::NavigationInterface* getNavigation(QString const& id) override;
@@ -170,11 +170,12 @@ public:
 
     Scope::Ptr findTempScope(QString const& id) const;
 
-    bool loginToAccount(QString const& scope_id, QString const& service_name, QString const& service_type, QString const& provider_name);
+    void setSearchQueryString(const QString& search_query);
 
 public Q_SLOTS:
     void invalidateResults();
     virtual void dispatchSearch();
+    void setSearchInProgress(bool searchInProgress);
 
 Q_SIGNALS:
     void resultsDirtyChanged();
@@ -185,13 +186,12 @@ private Q_SLOTS:
     void typingFinished();
     void flushUpdates(bool finalize = false);
     void metadataRefreshed();
-    void internetFlagChanged(QString const& key);
     void departmentModelDestroyed(QObject* obj);
+    void previewStackDestroyed(QObject *obj);
 
 protected:
     explicit Scope(scopes_ng::Scopes* parent);
 
-    void setSearchInProgress(bool searchInProgress);
     void setStatus(unity::shell::scopes::ScopeInterface::Status status);
     void invalidateLastSearch();
 
@@ -208,7 +208,9 @@ private:
     void setCurrentNavigationId(QString const& id);
     void setFilterState(unity::scopes::FilterState const& filterState);
     void processSearchChunk(PushEvent* pushEvent);
+    void setCannedQuery(unity::scopes::CannedQuery const& query);
     void executeCannedQuery(unity::scopes::CannedQuery const& query, bool allowDelayedActivation);
+    void handlePreviewUpdate(unity::scopes::Result::SPtr const& result, unity::scopes::PreviewWidgetList const& widgets);
 
     void processResultSet(QList<std::shared_ptr<unity::scopes::CategorisedResult>>& result_set);
 
@@ -223,6 +225,7 @@ private:
     QString m_currentNavigationId;
     QString m_currentAltNavigationId;
     QVariantMap m_customizations;
+    std::unique_ptr<unity::scopes::Variant> m_queryUserData;
     bool m_isActive;
     bool m_searchInProgress;
     bool m_resultsDirty;
@@ -244,7 +247,6 @@ private:
     unity::scopes::FilterState m_filterState;
     unity::scopes::FilterState m_receivedFilterState;
     unity::shell::scopes::ScopeInterface::Status m_status;
-    QGSettings* m_settings;
     QScopedPointer<SettingsModel> m_settingsModel;
     QSharedPointer<DepartmentNode> m_departmentTree;
     QSharedPointer<DepartmentNode> m_altNavTree;
@@ -260,6 +262,7 @@ private:
     QSharedPointer<LocationService> m_locationService;
     QSharedPointer<LocationService::Token> m_locationToken;
     QNetworkConfigurationManager m_network_manager;
+    QList<PreviewStack*> m_previewStacks;
 };
 
 } // namespace scopes_ng
