@@ -27,6 +27,7 @@
 
 // Qt
 #include <QDebug>
+#include <QGSettings>
 #include <QTimer>
 #include <QDBusConnection>
 #include <QProcess>
@@ -115,7 +116,7 @@ Scopes::Scopes(QObject *parent)
     connect(m_priv.get(), SIGNAL(safeInvalidateScopeResults(const QString&)), this,
             SLOT(invalidateScopeResults(const QString &)), Qt::QueuedConnection);
 
-    QDBusConnection::sessionBus().connect(QString(), QString("/com/canonical/unity/scopes"), QString("com.canonical.unity.scopes"), QString("InvalidateResults"), this, SLOT(invalidateScopeResults(QString)));
+    QDBusConnection::sessionBus().connect(QString(), QStringLiteral("/com/canonical/unity/scopes"), QStringLiteral("com.canonical.unity.scopes"), QStringLiteral("InvalidateResults"), this, SLOT(invalidateScopeResults(QString)));
 
     m_dashSettings = QGSettings::isSchemaInstalled("com.canonical.Unity.Dash") ? new QGSettings("com.canonical.Unity.Dash", QByteArray(), this) : nullptr;
     if (m_dashSettings)
@@ -123,10 +124,18 @@ Scopes::Scopes(QObject *parent)
         QObject::connect(m_dashSettings, &QGSettings::changed, this, &Scopes::dashSettingsChanged);
     }
 
-    m_overviewScope = new OverviewScope(this);
+    m_overviewScope = OverviewScope::newInstance(this);
+
+    m_registryRefreshTimer.setSingleShot(true);
+    connect(&m_registryRefreshTimer, SIGNAL(timeout()), this, SLOT(scopeRegistryChanged()));
+
     m_locationService.reset(new UbuntuLocationService());
 
     createUserAgentString();
+
+    m_scopesToDeleteTimer.setSingleShot(true);
+    m_scopesToDeleteTimer.setInterval(1000 * SCOPE_DELETE_DELAY);
+    connect(&m_scopesToDeleteTimer, SIGNAL(timeout()), SLOT(purgeScopesToDelete()));
 }
 
 Scopes::~Scopes()
@@ -142,6 +151,11 @@ QString Scopes::userAgentString() const
     return m_userAgent;
 }
 
+void Scopes::purgeScopesToDelete()
+{
+    m_scopesToDelete.clear();
+}
+
 int Scopes::rowCount(const QModelIndex& parent) const
 {
     Q_UNUSED(parent)
@@ -156,44 +170,10 @@ int Scopes::count() const
 
 void Scopes::createUserAgentString()
 {
-    QProcess *dpkg = new QProcess(this);
-    connect(dpkg, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(dpkgFinished()));
-    connect(dpkg, SIGNAL(error(QProcess::ProcessError)), this, SLOT(initPopulateScopes()));
-    dpkg->start("dpkg-query -W libunity-scopes3 unity-plugin-scopes unity8", QIODevice::ReadOnly);
-}
-
-void Scopes::dpkgFinished()
-{
-    QProcess *dpkg = qobject_cast<QProcess *>(sender());
-    if (dpkg) {
-        while (dpkg->canReadLine()) {
-            const QString line = dpkg->readLine();
-            const QStringList lineParts = line.split(QRegExp("\\s+"), QString::SkipEmptyParts);
-            QString key;
-            if (lineParts.size() == 2) {
-                if (lineParts.at(0).startsWith("libunity-scopes")) {
-                    key = "scopes-api";
-                }
-                else if (lineParts.at(0).startsWith("unity-plugin-scopes")) {
-                    key = "plugin";
-                }
-                else if (lineParts.at(0).startsWith("unity8")) {
-                    key = "unity8";
-                }
-                if (!key.isEmpty()) {
-                    m_versions.push_back(qMakePair(key, lineParts.at(1)));
-                } else {
-                    qWarning() << "Unexpected dpkg-query output:" << line;
-                }
-            }
-        }
-        dpkg->deleteLater();
-
-        QProcess *lsb_release = new QProcess(this);
-        connect(lsb_release, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(lsbReleaseFinished()));
-        connect(lsb_release, SIGNAL(error(QProcess::ProcessError)), this, SLOT(initPopulateScopes()));
-        lsb_release->start("lsb_release -r", QIODevice::ReadOnly);
-    }
+    QProcess *lsb_release = new QProcess(this);
+    connect(lsb_release, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(lsbReleaseFinished()));
+    connect(lsb_release, SIGNAL(error(QProcess::ProcessError)), this, SLOT(initPopulateScopes()));
+    lsb_release->start(QStringLiteral("lsb_release -r"), QIODevice::ReadOnly);
 }
 
 void Scopes::lsbReleaseFinished()
@@ -201,31 +181,50 @@ void Scopes::lsbReleaseFinished()
     QProcess *lsb_release = qobject_cast<QProcess *>(sender());
     if (lsb_release) {
         const QString out = lsb_release->readAllStandardOutput();
-        const QStringList parts = out.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+        const QStringList parts = out.split(QRegExp(QStringLiteral("\\s+")), QString::SkipEmptyParts);
         if (parts.size() == 2) {
-            m_versions.push_back(qMakePair(QString("release"), parts.at(1)));
+            m_versions.push_back(qMakePair(QStringLiteral("release"), parts.at(1)));
         }
         lsb_release->deleteLater();
-
-        QFile buildFile("/etc/ubuntu-build");
-        if (buildFile.open(QIODevice::ReadOnly)) {
-            QTextStream str(&buildFile);
-            QString bld;
-            str >> bld;
-            m_versions.push_back(qMakePair(QString("build"), bld));
-        }
-
-        const QString partnerId = readPartnerId();
-        if (!partnerId.isEmpty()) {
-            m_versions.push_back(qMakePair(QString("partner"), partnerId));
-        }
-
-        QUrlQuery q;
-        q.setQueryItems(m_versions);
-        m_versions.clear();
-        m_userAgent = q.toString();
     }
 
+    // map package version to a simple name we send to SSS
+    const QMap<QString, QString> versions({
+        {QStringLiteral("unity-plugin-scopes"), QStringLiteral("plugin")},
+        {QStringLiteral("unity8"), QStringLiteral("unity8")},
+        {QStringLiteral("libunity-scopes"), QStringLiteral("scopes-api")}});
+
+    // determine versions of unity8, unity-plugin-scopes and libunity-scopes1.0
+    for (QMap<QString, QString>::const_iterator pkg = versions.constBegin(); pkg != versions.constEnd(); pkg++) {
+        QFile versionFile("/var/lib/" + pkg.key() + "/version");
+        if (versionFile.open(QIODevice::ReadOnly)) {
+            QTextStream str(&versionFile);
+            QString ver;
+            str >> ver;
+            m_versions.push_back(qMakePair(pkg.value(), ver));
+        } else {
+            qWarning() << "Couldn't determine the version of" << pkg.key();
+        }
+    }
+
+    QFile buildFile(QStringLiteral("/etc/ubuntu-build"));
+    if (buildFile.open(QIODevice::ReadOnly)) {
+        QTextStream str(&buildFile);
+        QString bld;
+        str >> bld;
+        m_versions.push_back(qMakePair(QStringLiteral("build"), bld));
+    }
+
+    const QString partnerId = readPartnerId();
+    if (!partnerId.isEmpty()) {
+        m_versions.push_back(qMakePair(QStringLiteral("partner"), partnerId));
+    }
+
+    QUrlQuery q;
+    q.setQueryItems(m_versions);
+    m_versions.clear();
+    m_userAgent = q.toString();
+
     qDebug() << "User agent string:" << m_userAgent;
     initPopulateScopes();
 }
@@ -234,7 +233,7 @@ QString Scopes::readPartnerId()
 {
     // read /custom/partner-id value if present
     QString partnerId;
-    QFile partnerIdFile(PARTNER_ID_FILE);
+    QFile partnerIdFile(QStringLiteral(PARTNER_ID_FILE));
     if (partnerIdFile.exists())
     {
         if (partnerIdFile.open(QIODevice::ReadOnly))
@@ -244,7 +243,7 @@ QString Scopes::readPartnerId()
         }
         else
         {
-            qWarning() << "Cannot open" << QString(PARTNER_ID_FILE) << "for reading";
+            qWarning() << "Cannot open" << QStringLiteral(PARTNER_ID_FILE) << "for reading";
         }
     }
     return partnerId;
@@ -294,8 +293,8 @@ void Scopes::discoveryFinished()
         // add all visible scopes
         for (auto it = scopes.begin(); it != scopes.end(); ++it) {
             if (!it->second.invisible()) {
-                auto scope = new Scope(this);
-                connect(scope, SIGNAL(isActiveChanged()), this, SLOT(prepopulateNextScopes()));
+                Scope::Ptr scope = Scope::newInstance(this);
+                connect(scope.data(), SIGNAL(isActiveChanged()), this, SLOT(prepopulateNextScopes()));
                 scope->setScopeData(it->second);
                 m_scopes.append(scope);
             }
@@ -360,7 +359,7 @@ void Scopes::completeDiscoveryFinished()
 
 void Scopes::prepopulateNextScopes()
 {
-    for (QList<Scope*>::iterator it = m_scopes.begin(); it != m_scopes.end(); it++) {
+    for (auto it = m_scopes.begin(); it != m_scopes.end(); it++) {
         // query next two scopes following currently active scope
         if ((*it)->isActive()) {
             ++it;
@@ -368,7 +367,7 @@ void Scopes::prepopulateNextScopes()
                 auto scope = *(it++);
                 if (!scope->initialQueryDone()) {
                     qDebug() << "Pre-populating scope" << scope->id();
-                    scope->setSearchQuery("");
+                    scope->setSearchQuery(QLatin1String(""));
                     // must dispatch search explicitly since setSearchQuery will not do that for inactive scope
                     scope->dispatchSearch();
                 }
@@ -392,7 +391,7 @@ void Scopes::processFavoriteScopes()
     if (m_dashSettings) {
         QStringList newFavorites;
         QMap<QString, int> favScopesLut;
-        for (auto const& fv: m_dashSettings->get("favoriteScopes").toList())
+        for (auto const& fv: m_dashSettings->get(QStringLiteral("favoriteScopes")).toList())
         {
             int pos = 0;
             try
@@ -432,10 +431,12 @@ void Scopes::processFavoriteScopes()
             if (!favScopesLut.contains((*it)->id()))
             {
                 beginRemoveRows(QModelIndex(), row, row);
-                (*it)->setFavorite(false);
-                //
+                Scope::Ptr toDelete = *it;
+                toDelete->setFavorite(false);
                 // we need to delay actual deletion of Scope object so that shell can animate it
-                QTimer::singleShot(1000 * SCOPE_DELETE_DELAY, (*it), SLOT(deleteLater()));
+                m_scopesToDelete.push_back(toDelete);
+                // if the timer is already active, we just wait a bit longer, which is no problem
+                m_scopesToDeleteTimer.start();
                 it = m_scopes.erase(it);
                 endRemoveRows();
             }
@@ -457,8 +458,8 @@ void Scopes::processFavoriteScopes()
                 auto it = m_cachedMetadata.find(fav);
                 if (it != m_cachedMetadata.end())
                 {
-                    auto scope = new Scope(this);
-                    connect(scope, SIGNAL(isActiveChanged()), this, SLOT(prepopulateNextScopes()));
+                    Scope::Ptr scope = Scope::newInstance(this);
+                    connect(scope.data(), SIGNAL(isActiveChanged()), this, SLOT(prepopulateNextScopes()));
                     scope->setScopeData(*(it.value()));
                     scope->setFavorite(true);
                     beginInsertRows(QModelIndex(), row, row);
@@ -497,7 +498,7 @@ void Scopes::processFavoriteScopes()
 
 void Scopes::dashSettingsChanged(QString const& key)
 {
-    if (key != "favoriteScopes") {
+    if (key != QLatin1String("favoriteScopes")) {
         return;
     }
 
@@ -531,28 +532,20 @@ void Scopes::refreshFinished()
 void Scopes::invalidateScopeResults(QString const& scopeName)
 {
     // HACK! mediascanner invalidates local media scopes, but those are aggregated, so let's "forward" the call
-    if (scopeName == "mediascanner-music") {
-        invalidateScopeResults("musicaggregator");
-    } else if (scopeName == "mediascanner-video") {
-        invalidateScopeResults("videoaggregator");
-    } else if (scopeName == "scopes") {
+    if (scopeName == QLatin1String("mediascanner-music")) {
+        invalidateScopeResults(QStringLiteral("musicaggregator"));
+    } else if (scopeName == QLatin1String("mediascanner-video")) {
+        invalidateScopeResults(QStringLiteral("videoaggregator"));
+    } else if (scopeName == QLatin1String("scopes")) {
         // emitted when smart-scopes proxy or scope registry discovers new scopes
-        refreshScopeMetadata();
-        Q_FOREACH(Scope* scope, m_scopes) {
-            scope->invalidateResults();
-        }
+        m_registryRefreshTimer.start(5000);
         return;
     }
 
-    Scope* scope = getScopeById(scopeName);
+    auto scope = getScopeById(scopeName);
     if (scope == nullptr) {
         // check temporary scopes
-        for (auto s: m_scopes) {
-            scope = qobject_cast<Scope*>(s->findTempScope(scopeName));
-            if (scope) {
-                break;
-            }
-        }
+        scope = qobject_cast<Scope*>(findTempScope(scopeName));
     }
 
     if (scope) {
@@ -562,6 +555,19 @@ void Scopes::invalidateScopeResults(QString const& scopeName)
     }
 }
 
+void Scopes::scopeRegistryChanged()
+{
+    qDebug() << "Refreshing scope metadata";
+    refreshScopeMetadata();
+    Q_FOREACH(Scope::Ptr scope, m_scopes) {
+        scope->invalidateResults();
+    }
+
+    Q_FOREACH(Scope::Ptr scope, m_tempScopes) {
+        scope->invalidateResults();
+    }
+}
+
 QVariant Scopes::data(const QModelIndex& index, int role) const
 {
     int row = index.row();
@@ -572,11 +578,11 @@ QVariant Scopes::data(const QModelIndex& index, int role) const
         return QVariant();
     }
 
-    Scope* scope = m_scopes.at(index.row());
+    Scope::Ptr scope = m_scopes.at(index.row());
 
     switch (role) {
         case Scopes::RoleScope:
-            return QVariant::fromValue(scope);
+            return QVariant::fromValue(scope.data());
         case Scopes::RoleId:
             return QString(scope->id());
         case Scopes::RoleTitle:
@@ -587,27 +593,32 @@ QVariant Scopes::data(const QModelIndex& index, int role) const
 }
 
 unity::shell::scopes::ScopeInterface* Scopes::getScope(int row) const
+{
+    return getScopeByRow(row).data();
+}
+
+Scope::Ptr Scopes::getScopeByRow(int row) const
 {
     if (row >= m_scopes.size() || row < 0) {
-        return nullptr;
+        return Scope::Ptr();
     }
     return m_scopes[row];
 }
 
 unity::shell::scopes::ScopeInterface* Scopes::getScope(const QString& scopeId) const
 {
-    return getScopeById(scopeId);
+    return getScopeById(scopeId).data();
 }
 
-Scope* Scopes::getScopeById(QString const& scopeId) const
+Scope::Ptr Scopes::getScopeById(QString const& scopeId) const
 {
-    Q_FOREACH(Scope* scope, m_scopes) {
+    Q_FOREACH(Scope::Ptr scope, m_scopes) {
         if (scope->id() == scopeId) {
             return scope;
         }
     }
 
-    return nullptr;
+    return Scope::Ptr();
 }
 
 QStringList Scopes::getFavoriteIds() const
@@ -617,7 +628,7 @@ QStringList Scopes::getFavoriteIds() const
 
 void Scopes::setFavorite(QString const& scopeId, bool value)
 {
-    if (scopeId == CLICK_SCOPE_ID && !value)
+    if (scopeId == QStringLiteral(CLICK_SCOPE_ID) && !value)
     {
         qWarning() << "Cannot unfavorite" << scopeId;
         return;
@@ -647,11 +658,31 @@ void Scopes::setFavorite(QString const& scopeId, bool value)
         if (changed) {
             // update gsettings entry
             // note: this will trigger notification, so that new favorites are processed by processFavoriteScopes
-            m_dashSettings->set("favoriteScopes", QVariant(cannedQueries));
+            m_dashSettings->set(QStringLiteral("favoriteScopes"), QVariant(cannedQueries));
         }
     }
 }
 
+void Scopes::addTempScope(Scope::Ptr const& scope)
+{
+    m_tempScopes.insert(scope->id(), scope);
+}
+
+void Scopes::closeScope(unity::shell::scopes::ScopeInterface* scope)
+{
+    m_tempScopes.remove(scope->id());
+}
+
+Scope::Ptr Scopes::findTempScope(QString const& id) const
+{
+    auto it = m_tempScopes.find(id);
+    if (it != m_tempScopes.end())
+    {
+        return *it;
+    }
+    return Scope::Ptr();
+}
+
 void Scopes::moveFavoriteTo(QString const& scopeId, int index)
 {
     if (m_dashSettings)
@@ -680,7 +711,7 @@ void Scopes::moveFavoriteTo(QString const& scopeId, int index)
             cannedQueries.insert(index, query);
             // update gsettings entry
             // note: this will trigger notification, so that new favorites are processed by processFavoriteScopes
-            m_dashSettings->set("favoriteScopes", QVariant(cannedQueries));
+            m_dashSettings->set(QStringLiteral("favoriteScopes"), QVariant(cannedQueries));
         }
     }
 }
@@ -716,7 +747,17 @@ void Scopes::refreshScopeMetadata()
 
 unity::shell::scopes::ScopeInterface* Scopes::overviewScope() const
 {
-    return m_loaded ? m_overviewScope : nullptr;
+    return overviewScopeSPtr().data();
+}
+
+Scope::Ptr Scopes::overviewScopeSPtr() const
+{
+    Scope::Ptr result;
+    if (m_loaded)
+    {
+        result = m_overviewScope;
+    }
+    return result;
 }
 
 bool Scopes::loaded() const
